@@ -34,6 +34,22 @@ static inline void free_compat_syscall_stack(void *mem)
 				mem, ret);
 }
 
+struct syscall_args32 {
+	uint32_t nr, arg0, arg1, arg2, arg3, arg4, arg5;
+};
+
+static inline void do_full_int80(struct syscall_args32 *args)
+{
+	register unsigned long bp asm("bp") = args->arg5;
+	asm volatile ("int $0x80"
+		      : "+a" (args->nr),
+			"+b" (args->arg0), "+c" (args->arg1), "+d" (args->arg2),
+			"+S" (args->arg3), "+D" (args->arg4), "+r" (bp)
+			: : "r8", "r9", "r10", "r11");
+	args->arg5 = bp;
+}
+
+
 #ifdef CONFIG_COMPAT
 extern unsigned long call32_from_64(void *stack, void *func);
 #endif
