@@ -51,21 +51,21 @@ struct cr_options {
 	int			final_state;
 	char			*show_dump_file;
 	char			*show_fmt;
-	bool			check_extra_features;
-	bool			check_experimental_features;
+	int			check_extra_features;
+	int			check_experimental_features;
 	bool			show_pages_content;
 	union {
-		bool		restore_detach;
+		int		restore_detach;
 		bool		daemon_mode;
 	};
-	bool			restore_sibling;
+	int			restore_sibling;
 	bool			ext_unix_sk;
-	bool			shell_job;
-	bool			handle_file_locks;
-	bool			tcp_established_ok;
-	bool			evasive_devices;
-	bool			link_remap_ok;
-	bool			log_file_per_pid;
+	int			shell_job;
+	int			handle_file_locks;
+	int			tcp_established_ok;
+	int			evasive_devices;
+	int			link_remap_ok;
+	int			log_file_per_pid;
 	bool			swrk_restore;
 	char			*output;
 	char			*root;
@@ -76,15 +76,15 @@ struct cr_options {
 	struct list_head	external;
 	struct list_head	join_ns;
 	char			*libdir;
-	bool			use_page_server;
+	int			use_page_server;
 	unsigned short		port;
 	char			*addr;
 	int			ps_socket;
-	bool			track_mem;
+	int			track_mem;
 	char			*img_parent;
-	bool			auto_dedup;
+	int			auto_dedup;
 	unsigned int		cpu_cap;
-	bool			force_irmap;
+	int			force_irmap;
 	char			**exec_cmd;
 	unsigned int		manage_cgroups;
 	char			*new_global_cg_root;
@@ -92,8 +92,8 @@ struct cr_options {
 	char			*cgroup_props_file;
 	struct list_head	new_cgroup_roots;
 	bool			autodetect_ext_mounts;
-	bool			enable_external_sharing;
-	bool			enable_external_masters;
+	int			enable_external_sharing;
+	int			enable_external_masters;
 	bool			aufs;		/* auto-deteced, not via cli */
 	bool			overlayfs;
 #ifdef CONFIG_BINFMT_MISC_VIRTUALIZED
@@ -105,7 +105,7 @@ struct cr_options {
 	char			*lsm_profile;
 	unsigned int		timeout;
 	unsigned int		empty_ns;
-	bool			tcp_skip_in_flight;
+	int			tcp_skip_in_flight;
 	char			*work_dir;
 
 	/*
@@ -114,9 +114,9 @@ struct cr_options {
 	 * the deprecated stuff is not working, but it's still possible
 	 * to turn one ON while the code is in.
 	 */
-	bool			deprecated_ok;
-	bool			display_stats;
-	bool			weak_sysctls;
+	int			deprecated_ok;
+	int			display_stats;
+	int			weak_sysctls;
 	int			status_fd;
 	bool			orphan_pts_master;
 };
